
# Wrapper to enforce deterministic serialization
def deterministic_serialize_wrapper(orig_serialize_func):
    def wrapped(msg):
        return orig_serialize_func(msg, deterministic=True)
    return wrapped

# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from qm.grpc.qm.pb import job_results_pb2 as qm_dot_pb_dot_job__results__pb2

GRPC_GENERATED_VERSION = '1.68.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in qm/pb/job_results_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class JobResultsServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetJobResultSchema = channel.unary_unary(
                '/qm.grpc.results_analyser.JobResultsService/GetJobResultSchema',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaResponse.FromString,
                _registered_method=True)
        self.GetJobState = channel.unary_unary(
                '/qm.grpc.results_analyser.JobResultsService/GetJobState',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__results__pb2.GetJobStateRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobStateResponse.FromString,
                _registered_method=True)
        self.GetJobNamedResultHeader = channel.unary_unary(
                '/qm.grpc.results_analyser.JobResultsService/GetJobNamedResultHeader',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderResponse.FromString,
                _registered_method=True)
        self.GetJobNamedResult = channel.unary_stream(
                '/qm.grpc.results_analyser.JobResultsService/GetJobNamedResult',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__results__pb2.GetJobNamedResultRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultResponse.FromString,
                _registered_method=True)
        self.GetJobErrors = channel.unary_unary(
                '/qm.grpc.results_analyser.JobResultsService/GetJobErrors',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__results__pb2.GetJobErrorsRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobErrorsResponse.FromString,
                _registered_method=True)


class JobResultsServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetJobResultSchema(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobNamedResultHeader(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobNamedResult(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJobErrors(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_JobResultsServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetJobResultSchema': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJobResultSchema,
                    request_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaResponse.SerializeToString,
            ),
            'GetJobState': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJobState,
                    request_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobStateRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__results__pb2.GetJobStateResponse.SerializeToString,
            ),
            'GetJobNamedResultHeader': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJobNamedResultHeader,
                    request_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderResponse.SerializeToString,
            ),
            'GetJobNamedResult': grpc.unary_stream_rpc_method_handler(
                    servicer.GetJobNamedResult,
                    request_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__results__pb2.GetJobNamedResultResponse.SerializeToString,
            ),
            'GetJobErrors': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJobErrors,
                    request_deserializer=qm_dot_pb_dot_job__results__pb2.GetJobErrorsRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__results__pb2.GetJobErrorsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qm.grpc.results_analyser.JobResultsService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('qm.grpc.results_analyser.JobResultsService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class JobResultsService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetJobResultSchema(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.results_analyser.JobResultsService/GetJobResultSchema',
            qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaRequest.SerializeToString,
            qm_dot_pb_dot_job__results__pb2.GetJobResultSchemaResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetJobState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.results_analyser.JobResultsService/GetJobState',
            qm_dot_pb_dot_job__results__pb2.GetJobStateRequest.SerializeToString,
            qm_dot_pb_dot_job__results__pb2.GetJobStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetJobNamedResultHeader(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.results_analyser.JobResultsService/GetJobNamedResultHeader',
            qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderRequest.SerializeToString,
            qm_dot_pb_dot_job__results__pb2.GetJobNamedResultHeaderResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetJobNamedResult(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/qm.grpc.results_analyser.JobResultsService/GetJobNamedResult',
            qm_dot_pb_dot_job__results__pb2.GetJobNamedResultRequest.SerializeToString,
            qm_dot_pb_dot_job__results__pb2.GetJobNamedResultResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetJobErrors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.results_analyser.JobResultsService/GetJobErrors',
            qm_dot_pb_dot_job__results__pb2.GetJobErrorsRequest.SerializeToString,
            qm_dot_pb_dot_job__results__pb2.GetJobErrorsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



import abc
import threading
from collections import deque
from abc import abstractmethod
from types import TracebackType
from typing import TYPE_CHECKING, Set, Dict, List, Type, Deque, Tuple, Literal, Optional, cast

from qm.grpc.qm.pb import inc_qua_pb2
from qm.program.program import Program
from qm.exceptions import QmQuaException
from qm.api.models.capabilities import Capability

if TYPE_CHECKING:
    from qm.qua._dsl.streams.external_streams import QuaStreamDirection
    from qm.qua._dsl.stream_processing.stream_processing import ResultStreamSource, _OutputStream
    from qm.qua._dsl.stream_processing.direct_stream_processing_interface import DirectStreamSourceInterface

# The root level scopes stack. Intended for use exclusively by _get_scopes_stack().
# It is a thread-local stack - each thread gets its own stack, so it is safe for multi-threaded use.
_thread_local = threading.local()


def _get_scopes_stack() -> Deque["_BaseScope"]:
    """
    Get the scopes stack (thread-local), creates it if necessary.
    Intended for use exclusively by the base scope classes or the scopes manager.
    """
    if not hasattr(_thread_local, "scopes_stack"):
        _thread_local.scopes_stack = deque()
    return cast(Deque["_BaseScope"], _thread_local.scopes_stack)


class _BaseScope(abc.ABC):
    def __init__(self) -> None:
        self._statements: List[inc_qua_pb2.QuaProgram.AnyStatement] = []

    def __enter__(self) -> None:
        scopes_stack = _get_scopes_stack()
        if len(scopes_stack) == 0 and not isinstance(self, _ProgramScope):
            raise QmQuaException("First scope must be a program scope")

        scopes_stack.append(self)
        return None

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        # Let original exception propagate
        if exc_type is not None:
            return False

        self._on_clean_exit()
        return False

    def _on_clean_exit(self) -> None:
        scopes_stack = _get_scopes_stack()
        scopes_stack.pop()

    @property
    def statements(self) -> List[inc_qua_pb2.QuaProgram.AnyStatement]:
        return self._statements

    def append_statement(self, statement: inc_qua_pb2.QuaProgram.AnyStatement) -> None:
        self._statements.append(statement)


class _ProgramScope(_BaseScope):
    def __init__(self) -> None:
        super().__init__()
        self._program = Program()

        self.var_index = 0
        self.array_index = 0
        self.struct_index = 0
        self.result_index = 0

        self._declared_variables: List[inc_qua_pb2.QuaProgram.VarDeclaration] = []
        self._declared_streams: Dict[str, "ResultStreamSource"] = {}
        self._auto_processing_streams: List["DirectStreamSourceInterface"] = []  # type: ignore[type-arg]
        self._declared_input_streams: Set[str] = set()
        self._declared_external_streams: Dict[
            Tuple[int, "QuaStreamDirection"], inc_qua_pb2.QuaProgram.ExternalStreamDeclaration
        ] = {}

        self._used_capabilities: Set[Capability] = set()
        self._result_analysis_scope = _ResultAnalysisScope()

    def __enter__(self) -> Program:  # type: ignore[override]
        super().__enter__()
        self._program._set_in_scope()
        return self._program

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        self.auto_stream_processing()
        self._program._set_and_exit(self._generate_pb_qua_program(), self._used_capabilities)
        scopes_stack = _get_scopes_stack()
        scopes_stack.clear()
        return False

    @property
    def declared_streams(self) -> Dict[str, "ResultStreamSource"]:
        return self._declared_streams

    @property
    def declared_input_streams(self) -> Set[str]:
        return self._declared_input_streams

    @property
    def declared_external_streams(
        self,
    ) -> Dict[Tuple[int, "QuaStreamDirection"], inc_qua_pb2.QuaProgram.ExternalStreamDeclaration]:
        return self._declared_external_streams

    @property
    def result_analysis_scope(self) -> "_ResultAnalysisScope":
        return self._result_analysis_scope

    def add_var_declaration(self, declaration: inc_qua_pb2.QuaProgram.VarDeclaration) -> None:
        self._declared_variables.append(declaration)

    def add_stream_declaration(self, tag: str, stream: "ResultStreamSource") -> None:
        if tag in self._declared_streams:
            raise QmQuaException(f"Stream with tag {tag} already declared")
        self._declared_streams[tag] = stream

    def auto_stream_processing(self) -> None:
        """Process all auto-processing streams added to the program scope."""
        # to avoid calling similar stream processing multiple times in case of code duplication because of native loops
        current_processed_streams: set[str] = set()
        for stream in self._auto_processing_streams:
            stream.stream_processing(current_processed_streams)

    @property
    def auto_processing_streams(self) -> List["DirectStreamSourceInterface"]:  # type: ignore[type-arg]
        return self._auto_processing_streams

    def add_auto_processing_stream(self, stream: "DirectStreamSourceInterface") -> None:  # type: ignore[type-arg]
        self._auto_processing_streams.append(stream)

    def add_input_stream_declaration(self, var_name: str) -> None:
        self._declared_input_streams.add(var_name)

    def add_external_stream_declaration(
        self,
        stream_identifier: Tuple[int, "QuaStreamDirection"],
        stream_declaration: inc_qua_pb2.QuaProgram.ExternalStreamDeclaration,
    ) -> None:
        self._declared_external_streams[stream_identifier] = stream_declaration

    def add_used_capability(self, capability: Capability) -> None:
        self._used_capabilities.add(capability)

    def _generate_pb_qua_program(self) -> inc_qua_pb2.QuaProgram:
        return inc_qua_pb2.QuaProgram(
            script=inc_qua_pb2.QuaProgram.Script(
                variables=self._declared_variables,
                externalStreams=list(self._declared_external_streams.values()),
                body=inc_qua_pb2.QuaProgram.StatementsCollection(statements=self._statements),
            ),
            resultAnalysis=self._result_analysis_scope.generate_pb_result_analysis(),
        )


class _ResultAnalysisScope:
    def __enter__(self) -> None:
        return None

    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> Literal[False]:
        return False

    def __init__(self) -> None:
        self._saves: List[_OutputStream] = []

    def append_output_stream(self, output_stream: "_OutputStream") -> None:
        for _save in self._saves:
            if _save.tag == output_stream.tag:
                raise Exception("can not save two streams with the same tag")
        self._saves.append(output_stream)

    def generate_pb_result_analysis(self) -> inc_qua_pb2.QuaResultAnalysis:
        return inc_qua_pb2.QuaResultAnalysis(model=[output.to_proto() for output in self._saves])


class _LeadingScope(_BaseScope, abc.ABC):
    """
    Represents the first scope in a multi-part control structure.

    Examples include 'if', 'switch', or 'for' statements —
    constructs that initiate a compound control flow block.
    """

    def __init__(self, loc: str) -> None:
        super().__init__()
        self._loc = loc

    def _on_clean_exit(self) -> None:
        """Pops the current scope (created in __enter__) and appends its statement to the previous scope"""
        statement = self._create_statement()
        scopes_stack = _get_scopes_stack()
        scopes_stack.pop()
        scopes_stack[-1].append_statement(statement)

    @abc.abstractmethod
    def _create_statement(self) -> inc_qua_pb2.QuaProgram.AnyStatement:
        pass


class _FollowingScope(_BaseScope, abc.ABC):
    """
    Represents a scope that follows and complements a LeadingScope.

    Examples include 'else', 'case' —
    constructs that depend on a prior control block to make sense.
    """

    def _on_clean_exit(self) -> None:
        self._add_to_leading_scope()
        return super()._on_clean_exit()

    @abstractmethod
    def _add_to_leading_scope(self) -> None:
        raise NotImplementedError


class _LoopScope(_LeadingScope, abc.ABC):
    """Base class for loop scopes, providing common properties like name and size."""

    def __init__(self, loc: str) -> None:
        super().__init__(loc)
        self._name: Optional[str] = None
        self._size: Optional[int] = None
        self._averaged_streams: set[str] = set()

    @property
    def name(self) -> Optional[str]:
        return self._name

    @property
    def size(self) -> Optional[int]:
        return self._size

    def set_scope_metadata(self, name: str, size: int) -> None:
        self._name = name
        self._size = size

    def add_averaged_stream(self, stream_name: str) -> None:
        self._averaged_streams.add(stream_name)

    def averaged_streams(self) -> frozenset[str]:
        return frozenset(self._averaged_streams)


class _PythonNativeScope(_LoopScope):
    def __init__(self, loc: str) -> None:
        super().__init__(loc)
        self._current_value_idx: Optional[str] = None

    def _create_statement(self) -> inc_qua_pb2.QuaProgram.AnyStatement:
        raise QmQuaException("Native Python loops do not generate Qua statements")

    def _on_clean_exit(self) -> None:
        scopes_stack = _get_scopes_stack()
        scopes_stack.pop()
        for statement in self._statements:
            scopes_stack[-1].append_statement(statement)

    @property
    def current_value_name(self) -> str:
        if self._current_value_idx is None:
            raise QmQuaException("Current value name is not set for the native Python loop scope")
        return self._current_value_idx

    def set_current_iteration_number(self, idx: int) -> None:
        self._current_value_idx = str(idx)



from ._base_single_stream_fetcher import BaseSingleStreamFetcher
from ._single_stream_single_result_fetcher import SingleStreamSingleResultFetcher
from ._single_stream_multiple_results_fetcher import SingleStreamMultipleResultFetcher

__all__ = [
    "BaseSingleStreamFetcher",
    "SingleStreamSingleResultFetcher",
    "SingleStreamMultipleResultFetcher",
]



from qm.grpc.octave.v1 import api_pb2


def default_explore_response() -> api_pb2.ExploreResponse:
    return api_pb2.ExploreResponse(
        modules=[
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER, index=1),
                id="0025_F",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER, index=2),
                id="0024_F",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER, index=3),
                id="01170A01F2226006012",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER, index=4),
                id="0027_F",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_UPCONVERTER, index=5),
                id="01170A01F2215017712",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_DOWNCONVERTER, index=1),
                id="2",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_RF_DOWNCONVERTER, index=2),
                id="3",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_IF_DOWNCONVERTER, index=1),
                id="0005_A",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_IF_DOWNCONVERTER, index=2),
                id="0007_A",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_SYNTHESIZER, index=3),
                id="0008_B",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_SYNTHESIZER, index=4),
                id="22150030_B",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_SYNTHESIZER, index=5),
                id="0010_B",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_SYNTHESIZER, index=6),
                id="22150023_B",
            ),
            api_pb2.ExploreResponse.ModuleId(
                module=api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_MOTHERBOARD, index=1)
            ),
        ]
    )


def default_identify_response() -> api_pb2.IdentifyResponse:
    rf_up_converters = [
        api_pb2.RFUpConvIdentity(
            index=1,
            connectivity=api_pb2.RFUpConvIdentity.Connectivity(
                i_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(i_index=1)),
                q_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(q_index=1)),
                lo_input=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=3, output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_MAIN
                    )
                ),
            ),
            parameters=api_pb2.RFUpConvIdentity.Parameters(attn_1_db=0, attn_2_db=0),
        ),
        api_pb2.RFUpConvIdentity(
            index=2,
            connectivity=api_pb2.RFUpConvIdentity.Connectivity(
                i_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(i_index=2)),
                q_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(q_index=2)),
                lo_input=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=6,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SECONDARY,
                    )
                ),
            ),
            parameters=api_pb2.RFUpConvIdentity.Parameters(attn_1_db=0, attn_2_db=0),
        ),
        api_pb2.RFUpConvIdentity(
            index=3,
            connectivity=api_pb2.RFUpConvIdentity.Connectivity(
                i_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(i_index=3)),
                q_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(q_index=3)),
                lo_input=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=6, output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_MAIN
                    )
                ),
            ),
            parameters=api_pb2.RFUpConvIdentity.Parameters(attn_1_db=0, attn_2_db=0),
        ),
        api_pb2.RFUpConvIdentity(
            index=4,
            connectivity=api_pb2.RFUpConvIdentity.Connectivity(
                i_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(i_index=4)),
                q_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(q_index=4)),
                lo_input=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=5,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SECONDARY,
                    )
                ),
            ),
            parameters=api_pb2.RFUpConvIdentity.Parameters(attn_1_db=0, attn_2_db=0),
        ),
        api_pb2.RFUpConvIdentity(
            index=5,
            connectivity=api_pb2.RFUpConvIdentity.Connectivity(
                i_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(i_index=5)),
                q_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(q_index=5)),
                lo_input=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=5, output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_MAIN
                    )
                ),
            ),
            parameters=api_pb2.RFUpConvIdentity.Parameters(attn_1_db=0, attn_2_db=0),
        ),
    ]

    rf_down_converters = [
        api_pb2.RFDownConvIdentity(
            index=1,
            connectivity=api_pb2.RFDownConvIdentity.Connectivity(
                # debug_rf_input_1=None,
                # debug_rf_input_2=None,
                # debug_rf_input_3=None,
                # debug_rf_input_4=None,
                # debug_rf_input_5=None,
                rf_main_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(rf_in_index=1)),
                lo_input_1=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=4,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SECONDARY,
                    )
                ),
                lo_input_2=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=3,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SECONDARY,
                    ),
                ),
            ),
        ),
        api_pb2.RFDownConvIdentity(
            index=2,
            connectivity=api_pb2.RFDownConvIdentity.Connectivity(
                debug_rf_input_1=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=1)),
                debug_rf_input_2=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=2)),
                debug_rf_input_3=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=3)),
                debug_rf_input_4=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=4)),
                debug_rf_input_5=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=5)),
                rf_main_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(rf_in_index=2)),
                lo_input_1=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=4,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_MAIN,
                    )
                ),
                lo_input_2=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=4,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SYNTH,
                    ),
                ),
            ),
        ),
    ]

    if_down_converters = [
        api_pb2.IFDownConvIdentity(
            index=1,
            connectivity=api_pb2.IFDownConvIdentity.Connectivity(
                channel_1_input=api_pb2.IFSource(
                    rf_downconv_if_source=api_pb2.RFDownConvIFSource(
                        index=1, output_port=api_pb2.RFDownConvIFSource.OutputPort.OUTPUT_PORT_I
                    )
                ),
                channel_1_lo_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(if_lo_i_index=1)),
                channel_2_input=api_pb2.IFSource(
                    rf_downconv_if_source=api_pb2.RFDownConvIFSource(
                        index=1, output_port=api_pb2.RFDownConvIFSource.OutputPort.OUTPUT_PORT_Q
                    )
                ),
                channel_2_lo_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(if_lo_q_index=1)),
            ),
        ),
        api_pb2.IFDownConvIdentity(
            index=2,
            connectivity=api_pb2.IFDownConvIdentity.Connectivity(
                channel_1_input=api_pb2.IFSource(
                    rf_downconv_if_source=api_pb2.RFDownConvIFSource(
                        index=2, output_port=api_pb2.RFDownConvIFSource.OutputPort.OUTPUT_PORT_I
                    )
                ),
                channel_1_lo_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(if_lo_i_index=2)),
                channel_2_input=api_pb2.IFSource(
                    rf_downconv_if_source=api_pb2.RFDownConvIFSource(
                        index=2, output_port=api_pb2.RFDownConvIFSource.OutputPort.OUTPUT_PORT_Q
                    )
                ),
                channel_2_lo_input=api_pb2.IFSource(external_if_input=api_pb2.ExternalIFInput(if_lo_q_index=2)),
            ),
        ),
    ]

    synthesizers = [
        api_pb2.SynthIdentity(
            index=3,
            connectivity=api_pb2.SynthIdentity.Connectivity(
                main_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(lo_input_index=1)),
                secondary_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(demod_lo_input_index=1)),
            ),
            parameters=api_pb2.SynthIdentity.Parameters(
                low_frequency_filters=[
                    api_pb2.SynthIdentity.Parameters.LowFrequencyFilter(index=0, filter_1="1", filter_2="2")
                ],
                medium_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
                high_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
            ),
        ),
        api_pb2.SynthIdentity(
            index=6,
            connectivity=api_pb2.SynthIdentity.Connectivity(
                main_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(lo_input_index=3)),
                secondary_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(lo_input_index=2)),
            ),
            parameters=api_pb2.SynthIdentity.Parameters(
                low_frequency_filters=[
                    api_pb2.SynthIdentity.Parameters.LowFrequencyFilter(index=0, filter_1="1", filter_2="2")
                ],
                medium_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
                high_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
            ),
        ),
        api_pb2.SynthIdentity(
            index=5,
            connectivity=api_pb2.SynthIdentity.Connectivity(
                main_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(lo_input_index=5)),
                secondary_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(lo_input_index=4)),
            ),
            parameters=api_pb2.SynthIdentity.Parameters(
                low_frequency_filters=[
                    api_pb2.SynthIdentity.Parameters.LowFrequencyFilter(index=0, filter_1="1", filter_2="2")
                ],
                medium_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
                high_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
            ),
        ),
        api_pb2.SynthIdentity(
            index=4,
            connectivity=api_pb2.SynthIdentity.Connectivity(
                # secondary_lo_input=None,
                main_lo_input=api_pb2.RFSource(external_input=api_pb2.ExternalRFInput(demod_lo_input_index=2)),
            ),
            parameters=api_pb2.SynthIdentity.Parameters(
                low_frequency_filters=[
                    api_pb2.SynthIdentity.Parameters.LowFrequencyFilter(index=0, filter_1="1", filter_2="2")
                ],
                medium_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
                high_frequency_filter=api_pb2.SynthIdentity.Parameters.ParametrizedFilter(),
            ),
        ),
    ]

    panel_identity = api_pb2.PanelIdentity(
        rf_outputs=[
            api_pb2.PanelIdentity.RFOutput(
                index=1, source=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=1))
            ),
            api_pb2.PanelIdentity.RFOutput(
                index=2, source=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=2))
            ),
            api_pb2.PanelIdentity.RFOutput(
                index=3, source=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=3))
            ),
            api_pb2.PanelIdentity.RFOutput(
                index=4, source=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=4))
            ),
            api_pb2.PanelIdentity.RFOutput(
                index=5, source=api_pb2.RFSource(rf_up_conv_output=api_pb2.UpConvRFOutput(index=5))
            ),
        ],
        if_output_i=[api_pb2.IFSource(if_downconv_if_source=api_pb2.IFDownConvIFSource(index=0, channel_index=0))],
        if_output_q=[api_pb2.IFSource(if_downconv_if_source=api_pb2.IFDownConvIFSource(index=0, channel_index=1))],
        synth_outputs=[
            api_pb2.PanelIdentity.SynthOutput(
                index=1,
                source=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=3,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SYNTH,
                    )
                ),
            ),
            api_pb2.PanelIdentity.SynthOutput(
                index=2,
                source=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=6,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SYNTH,
                    )
                ),
            ),
            api_pb2.PanelIdentity.SynthOutput(
                index=3,
                source=api_pb2.RFSource(
                    synth_output=api_pb2.SynthRFOutput(
                        index=5,
                        output_port=api_pb2.SynthRFOutput.OutputPort.OUTPUT_PORT_SYNTH,
                    )
                ),
            ),
        ],
    )
    return api_pb2.IdentifyResponse(
        rf_up_converters=rf_up_converters,
        rf_down_converters=rf_down_converters,
        if_down_converters=if_down_converters,
        synthesizers=synthesizers,
        panel_identity=panel_identity,
    )



from pathlib import Path
from typing import Set, Union, Optional, FrozenSet

from qm.grpc.qm.pb import inc_qua_pb2
from qm.api.models.capabilities import Capability
from qm.exceptions import ProgramScopeAccessError
from qm.program._qua_config_schema import load_config
from qm.type_hinting.config_types import FullQuaConfig


class Program:
    def __init__(self, program: Optional[inc_qua_pb2.QuaProgram] = None) -> None:
        """
        The `Program` class encapsulates a `QuaProgram` protobuf object along with additional metadata.
        It provides properties to access the program's state, including the capabilities used and
        whether it is currently being edited (`is_in_scope`). Additionally, it includes methods to
        serialize and deserialize the `QuaProgram` to and from various formats.

        This class is used internally during construction (within `ProgramScope`) and externally
        as a read-only API for users. Protected methods (prefixed with `_`) are intended only
        for internal use during the building phase.
        """
        self._program = program
        self._used_capabilities: Set[Capability] = set()
        self._is_in_scope = False

    def _ensure_not_in_scope(self) -> None:
        if self._is_in_scope:
            raise ProgramScopeAccessError()

    # -------- Public API (for users) --------
    @property
    def qua_program(self) -> inc_qua_pb2.QuaProgram:
        self._ensure_not_in_scope()
        if self._program is None:
            # This should technically never occur, but since "_program" is defined as Optional in the constructor, this check is required to satisfy mypy.
            raise RuntimeError("Program is not set")
        return self._program

    @property
    def used_capabilities(self) -> FrozenSet[Capability]:
        self._ensure_not_in_scope()
        return frozenset(self._used_capabilities)

    def is_in_scope(self) -> bool:
        return self._is_in_scope

    def to_protobuf(self, config: FullQuaConfig) -> bytes:
        """
        Serialize the program to a protobuf binary.
        """
        self._ensure_not_in_scope()
        loaded_config = load_config(config)
        copy = inc_qua_pb2.QuaProgram()
        copy.CopyFrom(self.qua_program)
        copy.config.CopyFrom(loaded_config)
        return copy.SerializeToString()

    @classmethod
    def from_protobuf(cls, binary: bytes) -> "Program":
        """
        Deserialize the program from a protobuf binary.
        """
        program = inc_qua_pb2.QuaProgram()
        program.ParseFromString(binary)
        return cls(program=program)

    def to_file(self, path: Union[str, Path], config: FullQuaConfig) -> None:
        """
        Serialize the program to a protobuf binary and write it to a file.
        """
        self._ensure_not_in_scope()
        if isinstance(path, str):
            path = Path(path)
        path.write_bytes(self.to_protobuf(config))

    @classmethod
    def from_file(cls, path: Union[str, Path]) -> "Program":
        """
        Deserialize the program from a protobuf binary file.
        """
        if isinstance(path, str):
            path = Path(path)
        return cls.from_protobuf(path.read_bytes())

    # -------- Protected API (Internal Builder Methods) --------
    def _set_in_scope(self) -> None:
        self._is_in_scope = True

    def _set_and_exit(self, program: inc_qua_pb2.QuaProgram, used_capabilities: Set[Capability]) -> None:
        self._program = program
        self._used_capabilities = used_capabilities
        self._is_in_scope = False



from typing import Union, Optional

from qm.grpc.qm.pb import inc_qua_config_pb2
from qm.api.models.capabilities import OPX_FEM_IDX
from qm.program._dict_to_pb_converter.base_converter import BaseDictToPbConverter
from qm.exceptions import InvalidOctaveParameter, ConfigValidationException, OctaveConnectionAmbiguity
from qm.type_hinting.config_types import (
    LoopbackType,
    StandardPort,
    OctaveConfigType,
    PortReferenceType,
    OctaveRFInputConfigType,
    OctaveRFOutputConfigType,
    OctaveIfOutputsConfigType,
    OctaveSingleIfOutputConfigType,
)

ALLOWED_GAINS = {x / 2 for x in range(-40, 41)}

IF_OUT1_DEFAULT = "out1"
IF_OUT2_DEFAULT = "out2"


class OctaveConverter(BaseDictToPbConverter[OctaveConfigType, inc_qua_config_pb2.QuaConfig.Octave.Config]):
    def convert(self, input_data: OctaveConfigType) -> inc_qua_config_pb2.QuaConfig.Octave.Config:
        return self.octave_to_pb(input_data)

    def deconvert(self, output_data: inc_qua_config_pb2.QuaConfig.Octave.Config) -> OctaveConfigType:
        raise NotImplementedError("Conversion of the octave configuration to dictionary is not available.")

    def octave_to_pb(self, data: OctaveConfigType) -> inc_qua_config_pb2.QuaConfig.Octave.Config:
        connectivity = data.get("connectivity", None)
        if isinstance(connectivity, str):
            connectivity = (connectivity, OPX_FEM_IDX)
        loopbacks = self.get_octave_loopbacks(data.get("loopbacks", []))
        rf_modules = {
            k: self.rf_module_to_pb(self.standardize_connectivity_for_if_in(v, connectivity, k))
            for k, v in data.get("RF_outputs", {}).items()
        }
        rf_inputs = {k: self.rf_input_to_pb(v, k) for k, v in data.get("RF_inputs", {}).items()}
        if_outputs = self._octave_if_outputs_to_pb(
            self.standardize_connectivity_for_if_out(data.get("IF_outputs", {}), connectivity)
        )
        return inc_qua_config_pb2.QuaConfig.Octave.Config(
            loopbacks=loopbacks,
            rf_outputs=rf_modules,
            rf_inputs=rf_inputs,
            if_outputs=if_outputs,
        )

    @staticmethod
    def get_octave_loopbacks(data: list[LoopbackType]) -> list[inc_qua_config_pb2.QuaConfig.Octave.Loopback]:
        loopbacks = [
            inc_qua_config_pb2.QuaConfig.Octave.Loopback(
                lo_source_input=getattr(inc_qua_config_pb2.QuaConfig.Octave.LoopbackInput, loopback[1]),
                lo_source_generator=inc_qua_config_pb2.QuaConfig.Octave.SynthesizerPort(
                    device_name=loopback[0][0],
                    port_name=getattr(
                        inc_qua_config_pb2.QuaConfig.Octave.SynthesizerOutputName, loopback[0][1].lower()
                    ),
                ),
            )
            for loopback in data
        ]
        return loopbacks

    @staticmethod
    def standardize_connectivity_for_if_in(
        data: OctaveRFOutputConfigType, controller_connectivity: Optional[tuple[str, int]], module_number: int
    ) -> OctaveRFOutputConfigType:
        if controller_connectivity is not None:
            if ("I_connection" in data) or ("Q_connection" in data):
                raise OctaveConnectionAmbiguity()

            data["I_connection"] = controller_connectivity + (2 * module_number - 1,)
            data["Q_connection"] = controller_connectivity + (2 * module_number,)
        return data

    @staticmethod
    def standardize_connectivity_for_if_out(
        data: OctaveIfOutputsConfigType, controller_connectivity: Optional[tuple[str, int]]
    ) -> OctaveIfOutputsConfigType:
        if controller_connectivity is not None:
            if "IF_out1" not in data:
                data["IF_out1"] = {"name": IF_OUT1_DEFAULT}
            if "IF_out2" not in data:
                data["IF_out2"] = {"name": IF_OUT2_DEFAULT}
            if ("port" in data["IF_out1"]) or ("port" in data["IF_out2"]):
                raise OctaveConnectionAmbiguity()
            data["IF_out1"]["port"] = controller_connectivity + (1,)
            data["IF_out2"]["port"] = controller_connectivity + (2,)
        return data

    @staticmethod
    def _get_lo_frequency(data: Union[OctaveRFOutputConfigType, OctaveRFInputConfigType]) -> float:
        if "LO_frequency" not in data:
            raise ConfigValidationException("No LO frequency was set for upconverter")
        lo_freq = data["LO_frequency"]
        if not 2e9 <= lo_freq <= 18e9:
            raise ConfigValidationException(f"LO frequency {lo_freq} is out of range")
        return lo_freq

    def rf_module_to_pb(self, data: OctaveRFOutputConfigType) -> inc_qua_config_pb2.QuaConfig.Octave.RFOutputConfig:
        input_attenuators = data.get("input_attenuators", "OFF").upper()
        if input_attenuators not in {"ON", "OFF"}:
            raise ConfigValidationException("input_attenuators must be either ON or OFF")
        if "gain" not in data:
            raise ConfigValidationException("No gain was set for upconverter")
        gain = float(data["gain"])
        if gain not in ALLOWED_GAINS:
            raise ConfigValidationException(
                f"Gain should be an integer or half-integer between -20 and 20, got {gain})"
            )
        output_mode = getattr(
            inc_qua_config_pb2.QuaConfig.Octave.OutputSwitchState, data.get("output_mode", "always_off").lower()
        )
        lo_source = getattr(
            inc_qua_config_pb2.QuaConfig.Octave.LOSourceInput, data.get("LO_source", "internal").lower()
        )
        to_return = inc_qua_config_pb2.QuaConfig.Octave.RFOutputConfig(
            LO_frequency=self._get_lo_frequency(data),
            LO_source=lo_source,
            output_mode=output_mode,
            gain=gain,
            input_attenuators=input_attenuators == "ON",
        )
        if "I_connection" in data:
            to_return.I_connection.CopyFrom(dac_port_ref_to_pb(*_get_port_reference_with_fem(data["I_connection"])))
        if "Q_connection" in data:
            to_return.Q_connection.CopyFrom(dac_port_ref_to_pb(*_get_port_reference_with_fem(data["Q_connection"])))
        return to_return

    def rf_input_to_pb(
        self, data: OctaveRFInputConfigType, input_idx: int = 0
    ) -> inc_qua_config_pb2.QuaConfig.Octave.RFInputConfig:
        input_idx_to_default_lo_source = {0: "not_set", 1: "internal", 2: "external"}  # 0 here is just for the default
        rf_source = getattr(
            inc_qua_config_pb2.QuaConfig.Octave.DownconverterRFSource, data.get("RF_source", "RF_in").lower()
        )
        if input_idx == 1 and rf_source != inc_qua_config_pb2.QuaConfig.Octave.DownconverterRFSource.rf_in:
            raise InvalidOctaveParameter("Downconverter 1 must be connected to RF-in")

        lo_source = getattr(
            inc_qua_config_pb2.QuaConfig.Octave.LOSourceInput,
            data.get("LO_source", input_idx_to_default_lo_source[input_idx]).lower(),
        )
        if input_idx == 2 and lo_source == inc_qua_config_pb2.QuaConfig.Octave.LOSourceInput.internal:
            raise InvalidOctaveParameter("Downconverter 2 does not have internal LO")

        to_return = inc_qua_config_pb2.QuaConfig.Octave.RFInputConfig(
            RF_source=rf_source,
            LO_frequency=self._get_lo_frequency(data),
            LO_source=lo_source,
            IF_mode_I=getattr(inc_qua_config_pb2.QuaConfig.Octave.IFMode, data.get("IF_mode_I", "direct").lower()),
            IF_mode_Q=getattr(inc_qua_config_pb2.QuaConfig.Octave.IFMode, data.get("IF_mode_Q", "direct").lower()),
        )
        return to_return

    @staticmethod
    def single_if_output_to_pb(
        data: OctaveSingleIfOutputConfigType,
    ) -> inc_qua_config_pb2.QuaConfig.Octave.SingleIFOutputConfig:
        controller, fem, number = _get_port_reference_with_fem(data["port"])
        return inc_qua_config_pb2.QuaConfig.Octave.SingleIFOutputConfig(
            port=inc_qua_config_pb2.QuaConfig.AdcPortReference(controller=controller, fem=fem, number=number),
            name=data["name"],
        )

    def _octave_if_outputs_to_pb(
        self, data: OctaveIfOutputsConfigType
    ) -> inc_qua_config_pb2.QuaConfig.Octave.IFOutputsConfig:
        inst = inc_qua_config_pb2.QuaConfig.Octave.IFOutputsConfig()
        if "IF_out1" in data:
            inst.IF_out1.CopyFrom(self.single_if_output_to_pb(data["IF_out1"]))
        if "IF_out2" in data:
            inst.IF_out2.CopyFrom(self.single_if_output_to_pb(data["IF_out2"]))
        return inst


def dac_port_ref_to_pb(controller: str, fem: int, number: int) -> inc_qua_config_pb2.QuaConfig.DacPortReference:
    return inc_qua_config_pb2.QuaConfig.DacPortReference(controller=controller, fem=fem, number=number)


def _get_port_reference_with_fem(reference: PortReferenceType) -> StandardPort:
    if len(reference) == 2:
        return reference[0], OPX_FEM_IDX, reference[1]
    else:
        return reference



from abc import ABC, abstractmethod
from typing import Generic, Optional, Sequence

from qm.exceptions import QmQuaException
from qm.qua._scope_management._core_scopes import _LoopScope
from qm.qua._scope_management.scopes_manager import scopes_manager
from qm.qua.extensions.qua_iterators.qua_iterators_types import V, MetaDataType


class IterableBase(ABC, Generic[V]):
    def __init__(self, name: str, metadata: Optional[MetaDataType] = None):
        self._name = name
        self._metadata = metadata if metadata is not None else {}
        self._averaged_streams: frozenset[str] = frozenset()

    @property
    def name(self) -> str:
        return self._name

    @property
    def metadata(self) -> MetaDataType:
        return self._metadata

    # Return type intentionally omitted: subclasses declare specific return types
    # (QuaIteratorType[V], NativeIteratorType[V], MultiIteratorType). Annotating the
    # base with the IteratorType union (which collapses to Iterator[Any]) prevents
    # IDEs from resolving the narrower subclass types.
    @abstractmethod
    def __iter__(self):  # type: ignore[no-untyped-def]
        pass

    def __len__(self) -> int:
        return len(self.values)

    @property
    def buffer_size(self) -> int:
        return len(self)

    @property
    @abstractmethod
    def values(self) -> Sequence[V]:
        """
        get iterator values
        """
        pass

    def _get_current_scope(self) -> _LoopScope:
        """
        get current scope
        """
        curr_scope = scopes_manager.current_scope
        if not isinstance(curr_scope, _LoopScope):
            raise QmQuaException("Iterables can only be a loop scope.")
        return curr_scope

    def _add_to_current_scope(self) -> None:
        curr_scope = self._get_current_scope()
        curr_scope.set_scope_metadata(self.name, len(self))

    def _set_averaged_streams(self) -> None:
        """
        set averaged streams from scope
        """
        curr_scope = self._get_current_scope()
        self._averaged_streams = curr_scope.averaged_streams()

    def is_stream_averaged(self, stream_name: str) -> bool:
        """
        Check if stream is averaged on this iterable
        """
        return stream_name in self._averaged_streams

    @property
    def is_qua_iterable(self) -> bool:
        return False



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/pb/job_manager.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/pb/job_manager.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from qm.grpc.qm.pb import errors_pb2 as qm_dot_pb_dot_errors__pb2
from qm.grpc.qm.pb import general_messages_pb2 as qm_dot_pb_dot_general__messages__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17qm/pb/job_manager.proto\x12\x13qm.grpc.job_manager\x1a\x12qm/pb/errors.proto\x1a\x1cqm/pb/general_messages.proto\"v\n\x1bGetElementCorrectionRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x0e\n\x06qeName\x18\x02 \x01(\t\x12\x38\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.MatrixB\x02\x18\x01\"\xa5\x01\n\x1cGetElementCorrectionResponse\x12O\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32-.qm.grpc.job_manager.JobManagerResponseHeader\x12\x34\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"r\n\x1bSetElementCorrectionRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x0e\n\x06qeName\x18\x02 \x01(\t\x12\x34\n\ncorrection\x18\x03 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"\xa5\x01\n\x1cSetElementCorrectionResponse\x12O\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32-.qm.grpc.job_manager.JobManagerResponseHeader\x12\x34\n\ncorrection\x18\x02 \x01(\x0b\x32 .qm.grpc.general_messages.Matrix\"\xbc\x01\n\x18JobManagerResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\r\n\x05jobId\x18\x02 \x01(\t\x12\x41\n\x13jobManagerErrorType\x18\n \x01(\x0e\x32$.qm.grpc.errors.JobManagerErrorTypes\x12=\n\x0fjobErrorDetails\x18\x0b \x01(\x0b\x32$.qm.grpc.job_manager.JobErrorDetails\"\xcf\x01\n\x0fJobErrorDetails\x12W\n\x1djobOperationSpecificErrorType\x18\x01 \x01(\x0e\x32..qm.grpc.errors.JobOperationSpecificErrorTypesH\x00\x12\x45\n\x14\x63onfigQueryErrorType\x18\x02 \x01(\x0e\x32%.qm.grpc.errors.ConfigQueryErrorTypesH\x00\x12\x0f\n\x07message\x18\n \x01(\tB\x0b\n\terrorType\"j\n\x19JobOperationSpecificError\x12<\n\x04type\x18\x01 \x01(\x0e\x32..qm.grpc.errors.JobOperationSpecificErrorTypes\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x8f\x02\n\x18InsertInputStreamRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x12\n\nstreamName\x18\x02 \x01(\t\x12;\n\rintStreamData\x18\x03 \x01(\x0b\x32\".qm.grpc.job_manager.IntStreamDataH\x00\x12?\n\x0f\x66ixedStreamData\x18\x04 \x01(\x0b\x32$.qm.grpc.job_manager.FixedStreamDataH\x00\x12=\n\x0e\x62oolStreamData\x18\x05 \x01(\x0b\x32#.qm.grpc.job_manager.BoolStreamDataH\x00\x42\x13\n\x11stream_data_oneof\"\x91\x02\n ChangeOscillatorFrequencyRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x13\n\x0b\x65lementName\x18\x02 \x01(\t\x12\x16\n\x0enewFrequencyHz\x18\x03 \x01(\x01\x12g\n\x0fupdateComponent\x18\x04 \x01(\x0e\x32N.qm.grpc.job_manager.ChangeOscillatorFrequencyRequest.UpdateComponentSelection\"H\n\x18UpdateComponentSelection\x12\x08\n\x04\x62oth\x10\x00\x12\x0f\n\x0bupconverter\x10\x01\x12\x11\n\rdownconverter\x10\x02\"t\n!ChangeOscillatorFrequencyResponse\x12O\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32-.qm.grpc.job_manager.JobManagerResponseHeader\"\x1d\n\rIntStreamData\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x05\"\x1f\n\x0f\x46ixedStreamData\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x01\"\x1e\n\x0e\x42oolStreamData\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x08\"l\n\x19InsertInputStreamResponse\x12O\n\x18jobManagerResponseHeader\x18\x01 \x01(\x0b\x32-.qm.grpc.job_manager.JobManagerResponseHeader2\x96\x04\n\x11JobManagerService\x12}\n\x14GetElementCorrection\x12\x30.qm.grpc.job_manager.GetElementCorrectionRequest\x1a\x31.qm.grpc.job_manager.GetElementCorrectionResponse\"\x00\x12}\n\x14SetElementCorrection\x12\x30.qm.grpc.job_manager.SetElementCorrectionRequest\x1a\x31.qm.grpc.job_manager.SetElementCorrectionResponse\"\x00\x12t\n\x11InsertInputStream\x12-.qm.grpc.job_manager.InsertInputStreamRequest\x1a..qm.grpc.job_manager.InsertInputStreamResponse\"\x00\x12\x8c\x01\n\x19UpdateOscillatorFrequency\x12\x35.qm.grpc.job_manager.ChangeOscillatorFrequencyRequest\x1a\x36.qm.grpc.job_manager.ChangeOscillatorFrequencyResponse\"\x00\x42\x34\n\x12qm.grpc.jobManagerB\x16JobManagerServiceProtoP\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.pb.job_manager_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022qm.grpc.jobManagerB\026JobManagerServiceProtoP\001\242\002\003HLW'
  _globals['_GETELEMENTCORRECTIONREQUEST'].fields_by_name['correction']._loaded_options = None
  _globals['_GETELEMENTCORRECTIONREQUEST'].fields_by_name['correction']._serialized_options = b'\030\001'
  _globals['_GETELEMENTCORRECTIONREQUEST']._serialized_start=98
  _globals['_GETELEMENTCORRECTIONREQUEST']._serialized_end=216
  _globals['_GETELEMENTCORRECTIONRESPONSE']._serialized_start=219
  _globals['_GETELEMENTCORRECTIONRESPONSE']._serialized_end=384
  _globals['_SETELEMENTCORRECTIONREQUEST']._serialized_start=386
  _globals['_SETELEMENTCORRECTIONREQUEST']._serialized_end=500
  _globals['_SETELEMENTCORRECTIONRESPONSE']._serialized_start=503
  _globals['_SETELEMENTCORRECTIONRESPONSE']._serialized_end=668
  _globals['_JOBMANAGERRESPONSEHEADER']._serialized_start=671
  _globals['_JOBMANAGERRESPONSEHEADER']._serialized_end=859
  _globals['_JOBERRORDETAILS']._serialized_start=862
  _globals['_JOBERRORDETAILS']._serialized_end=1069
  _globals['_JOBOPERATIONSPECIFICERROR']._serialized_start=1071
  _globals['_JOBOPERATIONSPECIFICERROR']._serialized_end=1177
  _globals['_INSERTINPUTSTREAMREQUEST']._serialized_start=1180
  _globals['_INSERTINPUTSTREAMREQUEST']._serialized_end=1451
  _globals['_CHANGEOSCILLATORFREQUENCYREQUEST']._serialized_start=1454
  _globals['_CHANGEOSCILLATORFREQUENCYREQUEST']._serialized_end=1727
  _globals['_CHANGEOSCILLATORFREQUENCYREQUEST_UPDATECOMPONENTSELECTION']._serialized_start=1655
  _globals['_CHANGEOSCILLATORFREQUENCYREQUEST_UPDATECOMPONENTSELECTION']._serialized_end=1727
  _globals['_CHANGEOSCILLATORFREQUENCYRESPONSE']._serialized_start=1729
  _globals['_CHANGEOSCILLATORFREQUENCYRESPONSE']._serialized_end=1845
  _globals['_INTSTREAMDATA']._serialized_start=1847
  _globals['_INTSTREAMDATA']._serialized_end=1876
  _globals['_FIXEDSTREAMDATA']._serialized_start=1878
  _globals['_FIXEDSTREAMDATA']._serialized_end=1909
  _globals['_BOOLSTREAMDATA']._serialized_start=1911
  _globals['_BOOLSTREAMDATA']._serialized_end=1941
  _globals['_INSERTINPUTSTREAMRESPONSE']._serialized_start=1943
  _globals['_INSERTINPUTSTREAMRESPONSE']._serialized_end=2051
  _globals['_JOBMANAGERSERVICE']._serialized_start=2054
  _globals['_JOBMANAGERSERVICE']._serialized_end=2588
# @@protoc_insertion_point(module_scope)




# Wrapper to enforce deterministic serialization
def deterministic_serialize_wrapper(orig_serialize_func):
    def wrapped(msg):
        return orig_serialize_func(msg, deterministic=True)
    return wrapped

# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from qm.grpc.qm.pb import job_manager_pb2 as qm_dot_pb_dot_job__manager__pb2

GRPC_GENERATED_VERSION = '1.68.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in qm/pb/job_manager_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class JobManagerServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetElementCorrection = channel.unary_unary(
                '/qm.grpc.job_manager.JobManagerService/GetElementCorrection',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionResponse.FromString,
                _registered_method=True)
        self.SetElementCorrection = channel.unary_unary(
                '/qm.grpc.job_manager.JobManagerService/SetElementCorrection',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionResponse.FromString,
                _registered_method=True)
        self.InsertInputStream = channel.unary_unary(
                '/qm.grpc.job_manager.JobManagerService/InsertInputStream',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__manager__pb2.InsertInputStreamRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__manager__pb2.InsertInputStreamResponse.FromString,
                _registered_method=True)
        self.UpdateOscillatorFrequency = channel.unary_unary(
                '/qm.grpc.job_manager.JobManagerService/UpdateOscillatorFrequency',
                request_serializer=deterministic_serialize_wrapper(qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyRequest.SerializeToString),
                response_deserializer=qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyResponse.FromString,
                _registered_method=True)


class JobManagerServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetElementCorrection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetElementCorrection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InsertInputStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateOscillatorFrequency(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_JobManagerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetElementCorrection': grpc.unary_unary_rpc_method_handler(
                    servicer.GetElementCorrection,
                    request_deserializer=qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionResponse.SerializeToString,
            ),
            'SetElementCorrection': grpc.unary_unary_rpc_method_handler(
                    servicer.SetElementCorrection,
                    request_deserializer=qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionResponse.SerializeToString,
            ),
            'InsertInputStream': grpc.unary_unary_rpc_method_handler(
                    servicer.InsertInputStream,
                    request_deserializer=qm_dot_pb_dot_job__manager__pb2.InsertInputStreamRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__manager__pb2.InsertInputStreamResponse.SerializeToString,
            ),
            'UpdateOscillatorFrequency': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateOscillatorFrequency,
                    request_deserializer=qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyRequest.FromString,
                    response_serializer=qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qm.grpc.job_manager.JobManagerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('qm.grpc.job_manager.JobManagerService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class JobManagerService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetElementCorrection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.job_manager.JobManagerService/GetElementCorrection',
            qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionRequest.SerializeToString,
            qm_dot_pb_dot_job__manager__pb2.GetElementCorrectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetElementCorrection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.job_manager.JobManagerService/SetElementCorrection',
            qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionRequest.SerializeToString,
            qm_dot_pb_dot_job__manager__pb2.SetElementCorrectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InsertInputStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.job_manager.JobManagerService/InsertInputStream',
            qm_dot_pb_dot_job__manager__pb2.InsertInputStreamRequest.SerializeToString,
            qm_dot_pb_dot_job__manager__pb2.InsertInputStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UpdateOscillatorFrequency(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/qm.grpc.job_manager.JobManagerService/UpdateOscillatorFrequency',
            qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyRequest.SerializeToString,
            qm_dot_pb_dot_job__manager__pb2.ChangeOscillatorFrequencyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/grpc/v2/qmm_api.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/grpc/v2/qmm_api.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from qm.grpc.qm.grpc.v2 import common_types_pb2 as qm_dot_grpc_dot_v2_dot_common__types__pb2
from qm.grpc.qm.pb import compiler_pb2 as qm_dot_pb_dot_compiler__pb2
from qm.grpc.qm.pb import frontend_pb2 as qm_dot_pb_dot_frontend__pb2
from qm.grpc.qm.pb import inc_qua_pb2 as qm_dot_pb_dot_inc__qua__pb2
from qm.grpc.qm.pb import inc_qua_config_pb2 as qm_dot_pb_dot_inc__qua__config__pb2
from qm.grpc.qm.pb import qm_manager_pb2 as qm_dot_pb_dot_qm__manager__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18qm/grpc/v2/qmm_api.proto\x12\nqm.grpc.v2\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1dqm/grpc/v2/common_types.proto\x1a\x14qm/pb/compiler.proto\x1a\x14qm/pb/frontend.proto\x1a\x13qm/pb/inc_qua.proto\x1a\x1aqm/pb/inc_qua_config.proto\x1a\x16qm/pb/qm_manager.proto\"\x13\n\x11GetVersionRequest\"&\n$QmmServiceResetDataProcessingRequest\"8\n\"QmmServiceGetQuantumMachineRequest\x12\x12\n\nmachine_id\x18\x01 \x01(\t\":\n$QmmServiceCloseQuantumMachineRequest\x12\x12\n\nmachine_id\x18\x01 \x01(\t\"\x14\n\x12HealthCheckRequest\"\x88\x05\n\x13HealthCheckResponse\x12M\n\x07success\x18\x01 \x01(\x0b\x32:.qm.grpc.v2.HealthCheckResponse.HealthCheckResponseSuccessH\x00\x12I\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x38.qm.grpc.v2.HealthCheckResponse.HealthCheckResponseErrorH\x00\x1a\x97\x03\n\x1aHealthCheckResponseSuccess\x12\n\n\x02ok\x18\x02 \x01(\x08\x12^\n\ncomponents\x18\x03 \x03(\x0b\x32J.qm.grpc.v2.HealthCheckResponse.HealthCheckResponseSuccess.ComponentsEntry\x12X\n\x07\x64\x65tails\x18\x04 \x03(\x0b\x32G.qm.grpc.v2.HealthCheckResponse.HealthCheckResponseSuccess.DetailsEntry\x1am\n\x0f\x43omponentsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12I\n\x05value\x18\x02 \x01(\x0b\x32:.qm.grpc.v2.HealthCheckResponse.HealthCheckResponseSuccess:\x02\x38\x01\x1a\x44\n\x0c\x44\x65tailsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01\x1a+\n\x18HealthCheckResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\xe6\x01\n\x19OpenQuantumMachineRequest\x12-\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfig\x12\x43\n\nclose_mode\x18\x02 \x01(\x0e\x32/.qm.grpc.v2.OpenQuantumMachineRequest.CloseMode\"U\n\tCloseMode\x12\x1a\n\x16\x43LOSE_MODE_UNSPECIFIED\x10\x00\x12\x18\n\x14\x43LOSE_MODE_IF_NEEDED\x10\x01\x12\x12\n\x0e\x43LOSE_MODE_ALL\x10\x02\"\xa8\x04\n\x1aOpenQuantumMachineResponse\x12[\n\x07success\x18\x01 \x01(\x0b\x32H.qm.grpc.v2.OpenQuantumMachineResponse.OpenQuantumMachineResponseSuccessH\x00\x12W\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x46.qm.grpc.v2.OpenQuantumMachineResponse.OpenQuantumMachineResponseErrorH\x00\x1a|\n!OpenQuantumMachineResponseSuccess\x12\x1a\n\x12quantum_machine_id\x18\x01 \x01(\t\x12;\n\x10open_qm_warnings\x18\x02 \x03(\x0b\x32!.qm.grpc.qm_manager.OpenQmWarning\x1a\xc3\x01\n\x1fOpenQuantumMachineResponseError\x12M\n\x18\x63onfig_validation_errors\x18\x01 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessage\x12Q\n\x1aphysical_validation_errors\x18\x02 \x03(\x0b\x32-.qm.grpc.qm_manager.PhysicalValidationMessageB\x10\n\x0eresponse_oneof\"\xf9\x02\n\x19GetQuantumMachineResponse\x12Y\n\x07success\x18\x01 \x01(\x0b\x32\x46.qm.grpc.v2.GetQuantumMachineResponse.GetQuantumMachineResponseSuccessH\x00\x12U\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x44.qm.grpc.v2.GetQuantumMachineResponse.GetQuantumMachineResponseErrorH\x00\x1a\x65\n GetQuantumMachineResponseSuccess\x12\x12\n\nmachine_id\x18\x02 \x01(\t\x12-\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfig\x1a\x31\n\x1eGetQuantumMachineResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\x17\n\x15GetControllersRequest\" \n\x1eListOpenQuantumMachinesRequest\"\xf5\x02\n\x1fListOpenQuantumMachinesResponse\x12\x65\n\x07success\x18\x01 \x01(\x0b\x32R.qm.grpc.v2.ListOpenQuantumMachinesResponse.ListOpenQuantumMachinesResponseSuccessH\x00\x12\x61\n\x05\x65rror\x18\x02 \x01(\x0b\x32P.qm.grpc.v2.ListOpenQuantumMachinesResponse.ListOpenQuantumMachinesResponseErrorH\x00\x1a=\n&ListOpenQuantumMachinesResponseSuccess\x12\x13\n\x0bmachine_ids\x18\x01 \x03(\t\x1a\x37\n$ListOpenQuantumMachinesResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\" \n\x1e\x43loseAllQuantumMachinesRequest\"\x96\x01\n\x0fJobsQueryParams\x12\x1b\n\x13quantum_machine_ids\x18\x01 \x03(\t\x12\x0f\n\x07job_ids\x18\x02 \x03(\t\x12\x10\n\x08user_ids\x18\x03 \x03(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12.\n\x06status\x18\x05 \x03(\x0e\x32\x1e.qm.grpc.v2.JobExecutionStatus\"\xa8\x01\n\x0fJobResponseData\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12.\n\x06status\x18\x03 \x01(\x0e\x32\x1e.qm.grpc.v2.JobExecutionStatus\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.qm.grpc.v2.JobMetadata\x12\x15\n\ris_simulation\x18\x05 \x01(\x08\"\xa9\x01\n\x0bJobMetadata\x12.\n\ncreated_at\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nstarted_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_status_updated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"<\n\x0eGetJobsRequest\x12*\n\x05query\x18\x01 \x01(\x0b\x32\x1b.qm.grpc.v2.JobsQueryParams\"}\n\x0fGetJobsResponse\x12-\n\x07success\x18\x01 \x01(\x0b\x32\x1a.qm.grpc.v2.GetJobsSuccessH\x00\x12)\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x18.qm.grpc.v2.GetJobsErrorH\x00\x42\x10\n\x0eresponse_oneof\"\x1f\n\x0cGetJobsError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\";\n\x0eGetJobsSuccess\x12)\n\x04jobs\x18\x02 \x03(\x0b\x32\x1b.qm.grpc.v2.JobResponseData\"\xe5\x08\n\x16GetControllersResponse\x12S\n\x07success\x18\x01 \x01(\x0b\x32@.qm.grpc.v2.GetControllersResponse.GetControllersResponseSuccessH\x00\x12O\n\x05\x65rror\x18\x02 \x01(\x0b\x32>.qm.grpc.v2.GetControllersResponse.GetControllersResponseErrorH\x00\x1a\xf4\x01\n\x1dGetControllersResponseSuccess\x12m\n\x0f\x63ontrol_devices\x18\x01 \x03(\x0b\x32T.qm.grpc.v2.GetControllersResponse.GetControllersResponseSuccess.ControlDevicesEntry\x1a\x64\n\x13\x43ontrolDevicesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12<\n\x05value\x18\x02 \x01(\x0b\x32-.qm.grpc.v2.GetControllersResponse.Controller:\x02\x38\x01\x1a.\n\x1bGetControllersResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x92\x03\n\nController\x12\x10\n\x08hostname\x18\x01 \x01(\t\x12J\n\x0f\x63ontroller_type\x18\x02 \x01(\x0e\x32\x31.qm.grpc.v2.GetControllersResponse.ControllerType\x12\x45\n\x04\x66\x65ms\x18\x03 \x03(\x0b\x32\x37.qm.grpc.v2.GetControllersResponse.Controller.FemsEntry\x12U\n\x0ctemperatures\x18\x04 \x03(\x0b\x32?.qm.grpc.v2.GetControllersResponse.Controller.TemperaturesEntry\x1aS\n\tFemsEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.qm.grpc.v2.GetControllersResponse.Fem:\x02\x38\x01\x1a\x33\n\x11TemperaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\x1a?\n\x03\x46\x65m\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.qm.grpc.v2.GetControllersResponse.FemType\"E\n\x07\x46\x65mType\x12\x18\n\x14\x46\x45M_TYPE_UNSPECIFIED\x10\x00\x12\x0f\n\x0b\x46\x45M_TYPE_LF\x10\x01\x12\x0f\n\x0b\x46\x45M_TYPE_MW\x10\x02\"O\n\x0e\x43ontrollerType\x12\x1f\n\x1b\x43ONTROLLER_TYPE_UNSPECIFIED\x10\x00\x12\x1c\n\x18\x43ONTROLLER_TYPE_OPX_1000\x10\x01\x42\x10\n\x0eresponse_oneof\"\xab\x03\n\x12GetVersionResponse\x12K\n\x07success\x18\x01 \x01(\x0b\x32\x38.qm.grpc.v2.GetVersionResponse.GetVersionResponseSuccessH\x00\x12G\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x36.qm.grpc.v2.GetVersionResponse.GetVersionResponseErrorH\x00\x1a\xc0\x01\n\x19GetVersionResponseSuccess\x12\x0f\n\x07gateway\x18\x02 \x01(\t\x12^\n\x0b\x63ontrollers\x18\x03 \x03(\x0b\x32I.qm.grpc.v2.GetVersionResponse.GetVersionResponseSuccess.ControllersEntry\x1a\x32\n\x10\x43ontrollersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a*\n\x17GetVersionResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\x84\x02\n\x19QmmServiceSimulateRequest\x12-\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x1d.qm.grpc.qua_config.QuaConfig\x12\x33\n\x12high_level_program\x18\x03 \x01(\x0b\x32\x17.qm.grpc.qua.QuaProgram\x12=\n\x08simulate\x18\x04 \x01(\x0b\x32+.qm.grpc.frontend.ExecutionRequest.Simulate\x12\x44\n\x16\x63ontroller_connections\x18\x05 \x03(\x0b\x32$.qm.grpc.frontend.InterOpxConnection\"\x8e\x01\n\x1aQmmServiceSimulateResponse\x12\x30\n\x07success\x18\x01 \x01(\x0b\x32\x1d.qm.grpc.v2.SimulationSuccessH\x00\x12,\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1b.qm.grpc.v2.SimulationErrorH\x00\x42\x10\n\x0eresponse_oneof\"\x98\x01\n\x11SimulationSuccess\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x33\n\x08messages\x18\x02 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\x12>\n\tsimulated\x18\x03 \x01(\x0b\x32\'.qm.grpc.frontend.SimulatedResponsePartB\x02\x18\x01\"\xf9\x01\n\x0fSimulationError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x12M\n\x18\x63onfig_validation_errors\x18\x02 \x03(\x0b\x32+.qm.grpc.qm_manager.ConfigValidationMessage\x12Q\n\x1aphysical_validation_errors\x18\x03 \x03(\x0b\x32-.qm.grpc.qm_manager.PhysicalValidationMessage\x12\x33\n\x08messages\x18\x04 \x03(\x0b\x32!.qm.grpc.compiler.CompilerMessage\"\xc4\x02\n\x1bResetDataProcessingResponse\x12]\n\x07success\x18\x01 \x01(\x0b\x32J.qm.grpc.v2.ResetDataProcessingResponse.ResetDataProcessingResponseSuccessH\x00\x12Y\n\x05\x65rror\x18\x02 \x01(\x0b\x32H.qm.grpc.v2.ResetDataProcessingResponse.ResetDataProcessingResponseErrorH\x00\x1a$\n\"ResetDataProcessingResponseSuccess\x1a\x33\n ResetDataProcessingResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\xc4\x02\n\x1b\x43loseQuantumMachineResponse\x12]\n\x07success\x18\x01 \x01(\x0b\x32J.qm.grpc.v2.CloseQuantumMachineResponse.CloseQuantumMachineResponseSuccessH\x00\x12Y\n\x05\x65rror\x18\x02 \x01(\x0b\x32H.qm.grpc.v2.CloseQuantumMachineResponse.CloseQuantumMachineResponseErrorH\x00\x1a$\n\"CloseQuantumMachineResponseSuccess\x1a\x33\n CloseQuantumMachineResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\xe0\x02\n\x1f\x43loseAllQuantumMachinesResponse\x12\x65\n\x07success\x18\x01 \x01(\x0b\x32R.qm.grpc.v2.CloseAllQuantumMachinesResponse.CloseAllQuantumMachinesResponseSuccessH\x00\x12\x61\n\x05\x65rror\x18\x02 \x01(\x0b\x32P.qm.grpc.v2.CloseAllQuantumMachinesResponse.CloseAllQuantumMachinesResponseErrorH\x00\x1a(\n&CloseAllQuantumMachinesResponseSuccess\x1a\x37\n$CloseAllQuantumMachinesResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\x1b\n\x19\x43learAllJobResultsRequest\"\xbd\x02\n\x1a\x43learAllJobResultsResponse\x12[\n\x07success\x18\x01 \x01(\x0b\x32H.qm.grpc.v2.ClearAllJobResultsResponse.ClearAllJobResultsResponseSuccessH\x00\x12W\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x46.qm.grpc.v2.ClearAllJobResultsResponse.ClearAllJobResultsResponseErrorH\x00\x1a#\n!ClearAllJobResultsResponseSuccess\x1a\x32\n\x1f\x43learAllJobResultsResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof2\xc7\x08\n\nQmmService\x12M\n\nGetVersion\x12\x1d.qm.grpc.v2.GetVersionRequest\x1a\x1e.qm.grpc.v2.GetVersionResponse\"\x00\x12P\n\x0bHealthCheck\x12\x1e.qm.grpc.v2.HealthCheckRequest\x1a\x1f.qm.grpc.v2.HealthCheckResponse\"\x00\x12Y\n\x0eGetControllers\x12!.qm.grpc.v2.GetControllersRequest\x1a\".qm.grpc.v2.GetControllersResponse\"\x00\x12\x65\n\x12OpenQuantumMachine\x12%.qm.grpc.v2.OpenQuantumMachineRequest\x1a&.qm.grpc.v2.OpenQuantumMachineResponse\"\x00\x12l\n\x11GetQuantumMachine\x12..qm.grpc.v2.QmmServiceGetQuantumMachineRequest\x1a%.qm.grpc.v2.GetQuantumMachineResponse\"\x00\x12t\n\x17ListOpenQuantumMachines\x12*.qm.grpc.v2.ListOpenQuantumMachinesRequest\x1a+.qm.grpc.v2.ListOpenQuantumMachinesResponse\"\x00\x12t\n\x17\x43loseAllQuantumMachines\x12*.qm.grpc.v2.CloseAllQuantumMachinesRequest\x1a+.qm.grpc.v2.CloseAllQuantumMachinesResponse\"\x00\x12\x65\n\x12\x43learAllJobResults\x12%.qm.grpc.v2.ClearAllJobResultsRequest\x1a&.qm.grpc.v2.ClearAllJobResultsResponse\"\x00\x12\x44\n\x07GetJobs\x12\x1a.qm.grpc.v2.GetJobsRequest\x1a\x1b.qm.grpc.v2.GetJobsResponse\"\x00\x12r\n\x13ResetDataProcessing\x12\x30.qm.grpc.v2.QmmServiceResetDataProcessingRequest\x1a\'.qm.grpc.v2.ResetDataProcessingResponse\"\x00\x12[\n\x08Simulate\x12%.qm.grpc.v2.QmmServiceSimulateRequest\x1a&.qm.grpc.v2.QmmServiceSimulateResponse\"\x00\x42\x1a\n\x10qm.client.api.v2P\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.grpc.v2.qmm_api_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020qm.client.api.v2P\001\242\002\003HLW'
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_COMPONENTSENTRY']._loaded_options = None
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_COMPONENTSENTRY']._serialized_options = b'8\001'
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_DETAILSENTRY']._loaded_options = None
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_DETAILSENTRY']._serialized_options = b'8\001'
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS_CONTROLDEVICESENTRY']._loaded_options = None
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS_CONTROLDEVICESENTRY']._serialized_options = b'8\001'
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_FEMSENTRY']._loaded_options = None
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_FEMSENTRY']._serialized_options = b'8\001'
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_TEMPERATURESENTRY']._loaded_options = None
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_TEMPERATURESENTRY']._serialized_options = b'8\001'
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS_CONTROLLERSENTRY']._loaded_options = None
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS_CONTROLLERSENTRY']._serialized_options = b'8\001'
  _globals['_SIMULATIONSUCCESS'].fields_by_name['simulated']._loaded_options = None
  _globals['_SIMULATIONSUCCESS'].fields_by_name['simulated']._serialized_options = b'\030\001'
  _globals['_GETVERSIONREQUEST']._serialized_start=248
  _globals['_GETVERSIONREQUEST']._serialized_end=267
  _globals['_QMMSERVICERESETDATAPROCESSINGREQUEST']._serialized_start=269
  _globals['_QMMSERVICERESETDATAPROCESSINGREQUEST']._serialized_end=307
  _globals['_QMMSERVICEGETQUANTUMMACHINEREQUEST']._serialized_start=309
  _globals['_QMMSERVICEGETQUANTUMMACHINEREQUEST']._serialized_end=365
  _globals['_QMMSERVICECLOSEQUANTUMMACHINEREQUEST']._serialized_start=367
  _globals['_QMMSERVICECLOSEQUANTUMMACHINEREQUEST']._serialized_end=425
  _globals['_HEALTHCHECKREQUEST']._serialized_start=427
  _globals['_HEALTHCHECKREQUEST']._serialized_end=447
  _globals['_HEALTHCHECKRESPONSE']._serialized_start=450
  _globals['_HEALTHCHECKRESPONSE']._serialized_end=1098
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS']._serialized_start=628
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS']._serialized_end=1035
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_COMPONENTSENTRY']._serialized_start=856
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_COMPONENTSENTRY']._serialized_end=965
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_DETAILSENTRY']._serialized_start=967
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSESUCCESS_DETAILSENTRY']._serialized_end=1035
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSEERROR']._serialized_start=1037
  _globals['_HEALTHCHECKRESPONSE_HEALTHCHECKRESPONSEERROR']._serialized_end=1080
  _globals['_OPENQUANTUMMACHINEREQUEST']._serialized_start=1101
  _globals['_OPENQUANTUMMACHINEREQUEST']._serialized_end=1331
  _globals['_OPENQUANTUMMACHINEREQUEST_CLOSEMODE']._serialized_start=1246
  _globals['_OPENQUANTUMMACHINEREQUEST_CLOSEMODE']._serialized_end=1331
  _globals['_OPENQUANTUMMACHINERESPONSE']._serialized_start=1334
  _globals['_OPENQUANTUMMACHINERESPONSE']._serialized_end=1886
  _globals['_OPENQUANTUMMACHINERESPONSE_OPENQUANTUMMACHINERESPONSESUCCESS']._serialized_start=1546
  _globals['_OPENQUANTUMMACHINERESPONSE_OPENQUANTUMMACHINERESPONSESUCCESS']._serialized_end=1670
  _globals['_OPENQUANTUMMACHINERESPONSE_OPENQUANTUMMACHINERESPONSEERROR']._serialized_start=1673
  _globals['_OPENQUANTUMMACHINERESPONSE_OPENQUANTUMMACHINERESPONSEERROR']._serialized_end=1868
  _globals['_GETQUANTUMMACHINERESPONSE']._serialized_start=1889
  _globals['_GETQUANTUMMACHINERESPONSE']._serialized_end=2266
  _globals['_GETQUANTUMMACHINERESPONSE_GETQUANTUMMACHINERESPONSESUCCESS']._serialized_start=2096
  _globals['_GETQUANTUMMACHINERESPONSE_GETQUANTUMMACHINERESPONSESUCCESS']._serialized_end=2197
  _globals['_GETQUANTUMMACHINERESPONSE_GETQUANTUMMACHINERESPONSEERROR']._serialized_start=2199
  _globals['_GETQUANTUMMACHINERESPONSE_GETQUANTUMMACHINERESPONSEERROR']._serialized_end=2248
  _globals['_GETCONTROLLERSREQUEST']._serialized_start=2268
  _globals['_GETCONTROLLERSREQUEST']._serialized_end=2291
  _globals['_LISTOPENQUANTUMMACHINESREQUEST']._serialized_start=2293
  _globals['_LISTOPENQUANTUMMACHINESREQUEST']._serialized_end=2325
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE']._serialized_start=2328
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE']._serialized_end=2701
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE_LISTOPENQUANTUMMACHINESRESPONSESUCCESS']._serialized_start=2565
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE_LISTOPENQUANTUMMACHINESRESPONSESUCCESS']._serialized_end=2626
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE_LISTOPENQUANTUMMACHINESRESPONSEERROR']._serialized_start=2628
  _globals['_LISTOPENQUANTUMMACHINESRESPONSE_LISTOPENQUANTUMMACHINESRESPONSEERROR']._serialized_end=2683
  _globals['_CLOSEALLQUANTUMMACHINESREQUEST']._serialized_start=2703
  _globals['_CLOSEALLQUANTUMMACHINESREQUEST']._serialized_end=2735
  _globals['_JOBSQUERYPARAMS']._serialized_start=2738
  _globals['_JOBSQUERYPARAMS']._serialized_end=2888
  _globals['_JOBRESPONSEDATA']._serialized_start=2891
  _globals['_JOBRESPONSEDATA']._serialized_end=3059
  _globals['_JOBMETADATA']._serialized_start=3062
  _globals['_JOBMETADATA']._serialized_end=3231
  _globals['_GETJOBSREQUEST']._serialized_start=3233
  _globals['_GETJOBSREQUEST']._serialized_end=3293
  _globals['_GETJOBSRESPONSE']._serialized_start=3295
  _globals['_GETJOBSRESPONSE']._serialized_end=3420
  _globals['_GETJOBSERROR']._serialized_start=3422
  _globals['_GETJOBSERROR']._serialized_end=3453
  _globals['_GETJOBSSUCCESS']._serialized_start=3455
  _globals['_GETJOBSSUCCESS']._serialized_end=3514
  _globals['_GETCONTROLLERSRESPONSE']._serialized_start=3517
  _globals['_GETCONTROLLERSRESPONSE']._serialized_end=4642
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS']._serialized_start=3710
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS']._serialized_end=3954
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS_CONTROLDEVICESENTRY']._serialized_start=3854
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSESUCCESS_CONTROLDEVICESENTRY']._serialized_end=3954
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSEERROR']._serialized_start=3956
  _globals['_GETCONTROLLERSRESPONSE_GETCONTROLLERSRESPONSEERROR']._serialized_end=4002
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER']._serialized_start=4005
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER']._serialized_end=4407
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_FEMSENTRY']._serialized_start=4271
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_FEMSENTRY']._serialized_end=4354
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_TEMPERATURESENTRY']._serialized_start=4356
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLER_TEMPERATURESENTRY']._serialized_end=4407
  _globals['_GETCONTROLLERSRESPONSE_FEM']._serialized_start=4409
  _globals['_GETCONTROLLERSRESPONSE_FEM']._serialized_end=4472
  _globals['_GETCONTROLLERSRESPONSE_FEMTYPE']._serialized_start=4474
  _globals['_GETCONTROLLERSRESPONSE_FEMTYPE']._serialized_end=4543
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLERTYPE']._serialized_start=4545
  _globals['_GETCONTROLLERSRESPONSE_CONTROLLERTYPE']._serialized_end=4624
  _globals['_GETVERSIONRESPONSE']._serialized_start=4645
  _globals['_GETVERSIONRESPONSE']._serialized_end=5072
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS']._serialized_start=4818
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS']._serialized_end=5010
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS_CONTROLLERSENTRY']._serialized_start=4960
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSESUCCESS_CONTROLLERSENTRY']._serialized_end=5010
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSEERROR']._serialized_start=5012
  _globals['_GETVERSIONRESPONSE_GETVERSIONRESPONSEERROR']._serialized_end=5054
  _globals['_QMMSERVICESIMULATEREQUEST']._serialized_start=5075
  _globals['_QMMSERVICESIMULATEREQUEST']._serialized_end=5335
  _globals['_QMMSERVICESIMULATERESPONSE']._serialized_start=5338
  _globals['_QMMSERVICESIMULATERESPONSE']._serialized_end=5480
  _globals['_SIMULATIONSUCCESS']._serialized_start=5483
  _globals['_SIMULATIONSUCCESS']._serialized_end=5635
  _globals['_SIMULATIONERROR']._serialized_start=5638
  _globals['_SIMULATIONERROR']._serialized_end=5887
  _globals['_RESETDATAPROCESSINGRESPONSE']._serialized_start=5890
  _globals['_RESETDATAPROCESSINGRESPONSE']._serialized_end=6214
  _globals['_RESETDATAPROCESSINGRESPONSE_RESETDATAPROCESSINGRESPONSESUCCESS']._serialized_start=6107
  _globals['_RESETDATAPROCESSINGRESPONSE_RESETDATAPROCESSINGRESPONSESUCCESS']._serialized_end=6143
  _globals['_RESETDATAPROCESSINGRESPONSE_RESETDATAPROCESSINGRESPONSEERROR']._serialized_start=6145
  _globals['_RESETDATAPROCESSINGRESPONSE_RESETDATAPROCESSINGRESPONSEERROR']._serialized_end=6196
  _globals['_CLOSEQUANTUMMACHINERESPONSE']._serialized_start=6217
  _globals['_CLOSEQUANTUMMACHINERESPONSE']._serialized_end=6541
  _globals['_CLOSEQUANTUMMACHINERESPONSE_CLOSEQUANTUMMACHINERESPONSESUCCESS']._serialized_start=6434
  _globals['_CLOSEQUANTUMMACHINERESPONSE_CLOSEQUANTUMMACHINERESPONSESUCCESS']._serialized_end=6470
  _globals['_CLOSEQUANTUMMACHINERESPONSE_CLOSEQUANTUMMACHINERESPONSEERROR']._serialized_start=6472
  _globals['_CLOSEQUANTUMMACHINERESPONSE_CLOSEQUANTUMMACHINERESPONSEERROR']._serialized_end=6523
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE']._serialized_start=6544
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE']._serialized_end=6896
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE_CLOSEALLQUANTUMMACHINESRESPONSESUCCESS']._serialized_start=6781
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE_CLOSEALLQUANTUMMACHINESRESPONSESUCCESS']._serialized_end=6821
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE_CLOSEALLQUANTUMMACHINESRESPONSEERROR']._serialized_start=6823
  _globals['_CLOSEALLQUANTUMMACHINESRESPONSE_CLOSEALLQUANTUMMACHINESRESPONSEERROR']._serialized_end=6878
  _globals['_CLEARALLJOBRESULTSREQUEST']._serialized_start=6898
  _globals['_CLEARALLJOBRESULTSREQUEST']._serialized_end=6925
  _globals['_CLEARALLJOBRESULTSRESPONSE']._serialized_start=6928
  _globals['_CLEARALLJOBRESULTSRESPONSE']._serialized_end=7245
  _globals['_CLEARALLJOBRESULTSRESPONSE_CLEARALLJOBRESULTSRESPONSESUCCESS']._serialized_start=7140
  _globals['_CLEARALLJOBRESULTSRESPONSE_CLEARALLJOBRESULTSRESPONSESUCCESS']._serialized_end=7175
  _globals['_CLEARALLJOBRESULTSRESPONSE_CLEARALLJOBRESULTSRESPONSEERROR']._serialized_start=7177
  _globals['_CLEARALLJOBRESULTSRESPONSE_CLEARALLJOBRESULTSRESPONSEERROR']._serialized_end=7227
  _globals['_QMMSERVICE']._serialized_start=7248
  _globals['_QMMSERVICE']._serialized_end=8343
# @@protoc_insertion_point(module_scope)



from typing import Tuple, Union, Optional

from google.protobuf.empty_pb2 import Empty

from qm._loc import _get_loc
from qm.grpc.qm.pb import inc_qua_pb2
from qm.qua._scope_management.scopes_manager import scopes_manager
from qm.qua._expressions import Scalar, QuaVariable, to_scalar_pb_expression


def pause() -> None:
    """Pause the execution of the job until [qm.jobs.running_qm_job.RunningQmJob.resume][] is called.

    The quantum machines freezes on its current output state.
    """
    statement = inc_qua_pb2.QuaProgram.PauseStatement(loc=_get_loc(), qes=[])
    scopes_manager.append_statement(inc_qua_pb2.QuaProgram.AnyStatement(pause=statement))


def wait(duration: Scalar[int], *elements: str) -> None:
    r"""Wait for the given duration on all provided elements without outputting anything.
    Duration is in units of the clock cycle (4ns)

    Args:
        duration (Union[int,QUA variable of type int]): time to wait in
            units of the clock cycle (4ns). Range: [4, $2^{31}-1$]
            in steps of 1.
        *elements (Union[str,sequence of str]): elements to wait on

    Warning:

        In case the value of this is outside the range above, unexpected results may occur.

    Note:

        The purpose of the `wait` operation is to add latency. In most cases, the
        latency added will be exactly the same as that specified by the QUA variable or
        the literal used. However, in some cases an additional computational latency may
        be added. If the actual wait time has significance, such as in characterization
        experiments, the actual wait time should always be verified with a simulator.
    """
    loc = _get_loc()
    time = inc_qua_pb2.QuaProgram.AnyScalarExpression()
    time.CopyFrom(to_scalar_pb_expression(duration))
    statement = inc_qua_pb2.QuaProgram.WaitStatement(
        loc=loc,
        time=time,
        qe=[inc_qua_pb2.QuaProgram.QuantumElementReference(name=element, loc=loc) for element in elements],
    )
    scopes_manager.append_statement(inc_qua_pb2.QuaProgram.AnyStatement(wait=statement))


def wait_for_trigger(
    element: str,
    pulse_to_play: Optional[str] = None,
    trigger_element: Optional[Union[Tuple[str, str], str]] = None,
    time_tag_target: Optional[QuaVariable[int]] = None,
) -> None:
    """Wait for an external trigger on the provided element.

    During the command the OPX will play the pulse supplied by the ``pulse_to_play`` parameter

    Args:
        element (str): element to wait on
        pulse_to_play (str): the name of the pulse to play on the
            element while waiting for the external trigger. Must be a
            constant pulse. Default None, no pulse will be played.
        trigger_element (Union[str, tuple]): Available only with the
            OPD. The triggered element. See further details in the note.
        time_tag_target (QUA variable of type int): Available only with
            the OPD. The time at which the trigger arrived relative to
            the waiting start time. In ns.

    Warning:
        In the OPX - The maximum allowed voltage value for the digital trigger is 1.8V. A voltage higher than this can damage the
        controller.

        In the OPX+ and with the OPD - The maximum allowed voltage is 3.3V.

    Note:
        Read more about triggering with the OPD [here](../../Hardware/dib.md#wait-for-trigger)
    """
    time_tag_target_pb = None if time_tag_target is None else time_tag_target.unwrapped.variable

    loc = _get_loc()
    statement = inc_qua_pb2.QuaProgram.AnyStatement(
        waitForTrigger=inc_qua_pb2.QuaProgram.WaitForTriggerStatement(
            loc=loc,
            qe=[inc_qua_pb2.QuaProgram.QuantumElementReference(name=element, loc=loc)],
        )
    )

    if pulse_to_play is not None:
        statement.waitForTrigger.pulseToPlay.name = pulse_to_play
    if trigger_element is not None:
        if isinstance(trigger_element, tuple):
            el, out = trigger_element
            statement.waitForTrigger.elementOutput.CopyFrom(
                inc_qua_pb2.QuaProgram.WaitForTriggerStatement.ElementOutput(element=el, output=out)
            )
        else:
            statement.waitForTrigger.elementOutput.CopyFrom(
                inc_qua_pb2.QuaProgram.WaitForTriggerStatement.ElementOutput(element=trigger_element)
            )
    else:
        statement.waitForTrigger.globalTrigger.CopyFrom(Empty())
    if time_tag_target_pb is not None:
        statement.waitForTrigger.timeTagTarget.CopyFrom(time_tag_target_pb)

    scopes_manager.append_statement(statement)


def align(*elements: str) -> None:
    """Align several elements together.

    All the elements referenced in `elements` will wait for all the others to
    finish their currently running statement.

    If no arguments are given, the statement will align all the elements used in the program.

    Args:
        *elements (str): a single element, multiple elements, or none
    """
    loc = _get_loc()
    statement = inc_qua_pb2.QuaProgram.AlignStatement(
        loc=loc,
        qe=[inc_qua_pb2.QuaProgram.QuantumElementReference(name=element, loc=loc) for element in elements],
    )
    scopes_manager.append_statement(inc_qua_pb2.QuaProgram.AnyStatement(align=statement))



from qm.grpc.qm.pb import errors_pb2 as _errors_pb2
from qm.grpc.qm.pb import general_messages_pb2 as _general_messages_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetElementCorrectionRequest(_message.Message):
    __slots__ = ("jobId", "qeName", "correction")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    QENAME_FIELD_NUMBER: _ClassVar[int]
    CORRECTION_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    qeName: str
    correction: _general_messages_pb2.Matrix
    def __init__(self, jobId: _Optional[str] = ..., qeName: _Optional[str] = ..., correction: _Optional[_Union[_general_messages_pb2.Matrix, _Mapping]] = ...) -> None: ...

class GetElementCorrectionResponse(_message.Message):
    __slots__ = ("jobManagerResponseHeader", "correction")
    JOBMANAGERRESPONSEHEADER_FIELD_NUMBER: _ClassVar[int]
    CORRECTION_FIELD_NUMBER: _ClassVar[int]
    jobManagerResponseHeader: JobManagerResponseHeader
    correction: _general_messages_pb2.Matrix
    def __init__(self, jobManagerResponseHeader: _Optional[_Union[JobManagerResponseHeader, _Mapping]] = ..., correction: _Optional[_Union[_general_messages_pb2.Matrix, _Mapping]] = ...) -> None: ...

class SetElementCorrectionRequest(_message.Message):
    __slots__ = ("jobId", "qeName", "correction")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    QENAME_FIELD_NUMBER: _ClassVar[int]
    CORRECTION_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    qeName: str
    correction: _general_messages_pb2.Matrix
    def __init__(self, jobId: _Optional[str] = ..., qeName: _Optional[str] = ..., correction: _Optional[_Union[_general_messages_pb2.Matrix, _Mapping]] = ...) -> None: ...

class SetElementCorrectionResponse(_message.Message):
    __slots__ = ("jobManagerResponseHeader", "correction")
    JOBMANAGERRESPONSEHEADER_FIELD_NUMBER: _ClassVar[int]
    CORRECTION_FIELD_NUMBER: _ClassVar[int]
    jobManagerResponseHeader: JobManagerResponseHeader
    correction: _general_messages_pb2.Matrix
    def __init__(self, jobManagerResponseHeader: _Optional[_Union[JobManagerResponseHeader, _Mapping]] = ..., correction: _Optional[_Union[_general_messages_pb2.Matrix, _Mapping]] = ...) -> None: ...

class JobManagerResponseHeader(_message.Message):
    __slots__ = ("success", "jobId", "jobManagerErrorType", "jobErrorDetails")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    JOBID_FIELD_NUMBER: _ClassVar[int]
    JOBMANAGERERRORTYPE_FIELD_NUMBER: _ClassVar[int]
    JOBERRORDETAILS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    jobId: str
    jobManagerErrorType: _errors_pb2.JobManagerErrorTypes
    jobErrorDetails: JobErrorDetails
    def __init__(self, success: bool = ..., jobId: _Optional[str] = ..., jobManagerErrorType: _Optional[_Union[_errors_pb2.JobManagerErrorTypes, str]] = ..., jobErrorDetails: _Optional[_Union[JobErrorDetails, _Mapping]] = ...) -> None: ...

class JobErrorDetails(_message.Message):
    __slots__ = ("jobOperationSpecificErrorType", "configQueryErrorType", "message")
    JOBOPERATIONSPECIFICERRORTYPE_FIELD_NUMBER: _ClassVar[int]
    CONFIGQUERYERRORTYPE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    jobOperationSpecificErrorType: _errors_pb2.JobOperationSpecificErrorTypes
    configQueryErrorType: _errors_pb2.ConfigQueryErrorTypes
    message: str
    def __init__(self, jobOperationSpecificErrorType: _Optional[_Union[_errors_pb2.JobOperationSpecificErrorTypes, str]] = ..., configQueryErrorType: _Optional[_Union[_errors_pb2.ConfigQueryErrorTypes, str]] = ..., message: _Optional[str] = ...) -> None: ...

class JobOperationSpecificError(_message.Message):
    __slots__ = ("type", "message")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    type: _errors_pb2.JobOperationSpecificErrorTypes
    message: str
    def __init__(self, type: _Optional[_Union[_errors_pb2.JobOperationSpecificErrorTypes, str]] = ..., message: _Optional[str] = ...) -> None: ...

class InsertInputStreamRequest(_message.Message):
    __slots__ = ("jobId", "streamName", "intStreamData", "fixedStreamData", "boolStreamData")
    JOBID_FIELD_NUMBER: _ClassVar[int]
    STREAMNAME_FIELD_NUMBER: _ClassVar[int]
    INTSTREAMDATA_FIELD_NUMBER: _ClassVar[int]
    FIXEDSTREAMDATA_FIELD_NUMBER: _ClassVar[int]
    BOOLSTREAMDATA_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    streamName: str
    intStreamData: IntStreamData
    fixedStreamData: FixedStreamData
    boolStreamData: BoolStreamData
    def __init__(self, jobId: _Optional[str] = ..., streamName: _Optional[str] = ..., intStreamData: _Optional[_Union[IntStreamData, _Mapping]] = ..., fixedStreamData: _Optional[_Union[FixedStreamData, _Mapping]] = ..., boolStreamData: _Optional[_Union[BoolStreamData, _Mapping]] = ...) -> None: ...

class ChangeOscillatorFrequencyRequest(_message.Message):
    __slots__ = ("jobId", "elementName", "newFrequencyHz", "updateComponent")
    class UpdateComponentSelection(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        both: _ClassVar[ChangeOscillatorFrequencyRequest.UpdateComponentSelection]
        upconverter: _ClassVar[ChangeOscillatorFrequencyRequest.UpdateComponentSelection]
        downconverter: _ClassVar[ChangeOscillatorFrequencyRequest.UpdateComponentSelection]
    both: ChangeOscillatorFrequencyRequest.UpdateComponentSelection
    upconverter: ChangeOscillatorFrequencyRequest.UpdateComponentSelection
    downconverter: ChangeOscillatorFrequencyRequest.UpdateComponentSelection
    JOBID_FIELD_NUMBER: _ClassVar[int]
    ELEMENTNAME_FIELD_NUMBER: _ClassVar[int]
    NEWFREQUENCYHZ_FIELD_NUMBER: _ClassVar[int]
    UPDATECOMPONENT_FIELD_NUMBER: _ClassVar[int]
    jobId: str
    elementName: str
    newFrequencyHz: float
    updateComponent: ChangeOscillatorFrequencyRequest.UpdateComponentSelection
    def __init__(self, jobId: _Optional[str] = ..., elementName: _Optional[str] = ..., newFrequencyHz: _Optional[float] = ..., updateComponent: _Optional[_Union[ChangeOscillatorFrequencyRequest.UpdateComponentSelection, str]] = ...) -> None: ...

class ChangeOscillatorFrequencyResponse(_message.Message):
    __slots__ = ("jobManagerResponseHeader",)
    JOBMANAGERRESPONSEHEADER_FIELD_NUMBER: _ClassVar[int]
    jobManagerResponseHeader: JobManagerResponseHeader
    def __init__(self, jobManagerResponseHeader: _Optional[_Union[JobManagerResponseHeader, _Mapping]] = ...) -> None: ...

class IntStreamData(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, data: _Optional[_Iterable[int]] = ...) -> None: ...

class FixedStreamData(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, data: _Optional[_Iterable[float]] = ...) -> None: ...

class BoolStreamData(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, data: _Optional[_Iterable[bool]] = ...) -> None: ...

class InsertInputStreamResponse(_message.Message):
    __slots__ = ("jobManagerResponseHeader",)
    JOBMANAGERRESPONSEHEADER_FIELD_NUMBER: _ClassVar[int]
    jobManagerResponseHeader: JobManagerResponseHeader
    def __init__(self, jobManagerResponseHeader: _Optional[_Union[JobManagerResponseHeader, _Mapping]] = ...) -> None: ...



import warnings

from qm.simulate import SimulatorSamples, SimulatorControllerSamples

from ..utils import deprecation_message
from .._stream_results import (
    StreamsManager,
    StreamingResultFetcher,
    SingleStreamingResultFetcher,
    MultipleStreamingResultFetcher,
    SingleStreamSingleResultFetcher,
    SingleStreamMultipleResultFetcher,
)

warnings.warn(
    deprecation_message(
        "qm.results",
        "1.2.3",
        "2.0.0",
        "If you need anything from this module, import it directly from `qm` or from `qm.simulate` for simulator-related functionality.",
    ),
    DeprecationWarning,
)


__all__ = [
    "StreamsManager",
    "StreamingResultFetcher",
    "SingleStreamingResultFetcher",
    "MultipleStreamingResultFetcher",
    "SingleStreamSingleResultFetcher",
    "SingleStreamMultipleResultFetcher",
    "SimulatorSamples",
    "SimulatorControllerSamples",
]



import dataclasses
from typing import Optional

MAX_MESSAGE_SIZE = 1024 * 1024 * 100  # 100 mb in bytes
BASE_TIMEOUT = 60


@dataclasses.dataclass
class ConnectionDetails:
    host: str
    port: int
    credentials: Optional[str] = dataclasses.field(default=None)
    max_message_size: int = dataclasses.field(default=MAX_MESSAGE_SIZE)
    timeout: float = dataclasses.field(default=BASE_TIMEOUT)
